#!/bin/zsh

# Copyright 2014 - 2016 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

BR='
'

case $PROFILE in
medical)
  view_card="$_EXEC/templates/view_client.sh"
  edit_card="$_EXEC/templates/edit_client.sh"
  _GET[order]="${_GET[order]:-lastname}"
  _GET[filtertype]="${_GET[filtertype]:-name}"
  profile_medical=x
;;
circus)
  view_card="$_EXEC/templates/view_attendee.sh"
  edit_card="$_EXEC/templates/edit_attendee.sh"
  _GET[order]="${_GET[order]:-firstname}"
  _GET[filtertype]="${_GET[filtertype]:-any}"
  profile_circus=x
;;
esac

edit="${_GET[edit]}"
[ \! -f "vcard/$edit" -a \! -f "temp/$edit" ] && edit=''

listcourses() {
  ls -1 ${_DATA}/ical/*ics |while read file; do
    icstime="$(sed -rn 's:^DTSTART\:(TZID=.*\:)?([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z?\r$:\2-\3-\4 \5\:\6\:\7:p' "$file")"
    echo "$(date -d "$icstime" "+%u %H%M%S")\t$file"
  done |sort |sed -r 's:^.*\t(.*/)([^/]+)$:\2:'
}

list_hi_companies(){
  sed -rn 's;^X-HEALTH-INSURANCE:([^\;]+)\;.*$;\1;p' ${_DATA}/vcard/*vcf \
  | sort -u
}

listcards() {
  case "${_GET[filtertype]}" in
    any)
       grep -il "${_GET[filter]}" ${_DATA}/vcard/*vcf
      ;;
    name)
       egrep -xil "(FN|NICKNAME|N)(;.+)*:.*${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    street)
       egrep -xil "(ADR)(;.+)*:([^;]*;){2}${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    zip)
       egrep -xil "(ADR)(;.+)*:([^;]*;){5}${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    telephone)
       egrep -xil "(TEL)(;.+)*:.*${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    birth)
       egrep -xil "(BDAY)(;.+)*:${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    course)
      ;;
    *) ls -1 ${_DATA}/vcard/*vcf 2>/dev/null
      ;;
  esac |case "${_GET[order]}" in
    firstname)
      while read file; do
        fn=$(sed -rn 's:^N(;.+)*\:([^;]*;){1} *([^;]*).*$:\3:p' "$file")
	echo "$fn\t$file"
      done
      ;;
    lastname)
      while read file; do
        ln=$(sed -rn 's:^N(;.+)*\:([^;]*;){0} *([^;]*).*$:\3:p' "$file")
	echo "$ln\t$file"
      done
      ;;
    bdate)
      while read file; do
        bd=$(sed -rn 's:^BDAY(;.+)*\:(.*)$:\2:p' "$file")
	echo "$bd\t$file"
      done
      ;;
    *)
      sed -r 's:^.*$:x\t&:'
      ;;
  esac |sort |sed -r 's:^.*\t(.*/)([^/]+)$:\2:'
}

vcf_parse() {
  unset key
  sed -r ':X;N;$!bX; s;\r\n[ \t];;g; s;\r\n;\n;g;' "$1" \
  | sed -rn '
    # === turn property names to upper case, strip group names ===
    h; s;^([^;:]+);;;
    x; s;^([^;:\.]+\.)?([^;:]+).*$;\2;;
    y;abcdefghijklmnopqrstuvwxyz;ABCDEFGHIJKLMNOPQRSTUVWXYZ;
    G; s;\n;;;

    

    # === Normalise various known vendor properties ===
                s;^X-MS-CARDPICTURE(\;|:);PHOTO\1;;
                        s;^X-GENDER(\;|:);GENDER\1;;
                   s;^X-ANNIVERSARY(\;|:);ANNIVERSARY\1;;
         s;^X-EVOLUTION-ANNIVERSARY(\;|:);ANNIVERSARY\1;;
    s;^X-KADDRESSBOOK-X-ANNIVERSARY(\;|:);ANNIVERSARY\1;;
            s;^X-EVOLUTION-BLOG-URL(\;|:);URL\1;;
                              s;^AGENT(\;|:);RELATED\;VALUE=text\;TYPE=agent\1;;
                        s;^X-ASSISTANT(\;|:);RELATED\;VALUE=text\;TYPE=assistant\1;;
              s;^X-EVOLUTION-ASSISTANT(\;|:);RELATED\;VALUE=text\;TYPE=assistant\1;;
    s;^X-KADDRESSBOOK-X-ASSISTANTSNAME(\;|:);RELATED\;VALUE=text\;TYPE=assistant\1;;
                          s;^X-MANAGER(\;|:);RELATED\;VALUE=text\;TYPE=manager\1;;
                s;^X-EVOLUTION-MANAGER(\;|:);RELATED\;VALUE=text\;TYPE=manager\1;;
      s;^X-KADDRESSBOOK-X-MANAGERSNAME(\;|:);RELATED\;VALUE=text\;TYPE=manager\1;;
                           s;^X-SPOUSE(\;|:);RELATED\;VALUE=text\;TYPE=spouse\1;;
                 s;^X-EVOLUTION-SPOUSE(\;|:);RELATED\;VALUE=text\;TYPE=spouse\1;;
        s;^X-KADDRESSBOOK-X-SPOUSENAME(\;|:);RELATED\;VALUE=text\;TYPE=spouse\1;;

    # === Normalise obsolete vendor IM properties ===
            s;^X-AIM((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):;IMPP\1:aim:;;
            s;^X-ICQ((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):;IMPP\1:aim:;;
    s;^X-GOOGLE-TALK((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):;IMPP\1:xmpp:;;
         s;^X-JABBER((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):;IMPP\1:xmpp:;;
            s;^X-MSN((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):;IMPP\1:msn:;;
          s;^X-YAHOO((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):;IMPP\1:ymsgr:;;
            s;^X-SIP((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):(sip:)?;IMPP\1:sip:;;

    # === Update obsolete LABEL property ===
    s;^LABEL((\;[a-Z0-9-]+|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):(.*)$;ADR\1\;LABEL="\5":;;

    /^([A-Z0-9-]+)((\;[a-Z0-9-]+=?|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):(.*)$/{
      h;
      s;^([A-Z0-9-]+)((\;[a-Z0-9-]+=?|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):(.*)$;key='\''\1'\'';;
      H; g;

      s;^([A-Z0-9-]+)((\;[a-Z0-9-]+=?|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):([^\n]*)\n.*$;\2;;
      s;'\'';'\'\\\\\'\'';g
      s;\;([A-Z0-9-]+)(=|=(([^\;,:"]+|"[^"]+")(,[^\;,:"]+|,"[^"]+")*))?;\ntag[\1]='\''\3'\'';g
      s;^\n+;;; s;\n+$;;;
      /^.+$/H; g;

      s;^([A-Z0-9-]+)((\;[a-Z0-9-]+=?|\;[a-Z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):([^\n]*)\n.*$;\6;;
      # :X s;((^|[^\\])(\\\\)*)[;,];\1\n;g; s;((^|[^\\])(\\\\)*)\\([;,]);\4;g; tX;
      s;'\'';'\'\\\\\'\'';g
      s;^.*$;value='\''&'\'';
      H; g;

      s;^[^\n]*[\n ]+;;
      s;\n+$;;;

      p;
    }
    ' \
  | while read -r line; do
    declare -A tag
    case "$line" in
      value*) eval "$line";;
      tag*)   eval "$line";;
      key*)
        if [ -z "$key" ]; then
          eval "$line"
        else
          values[${key}]="${value//\\r\\n/$BR}"
          for t in ${(k)tag}; do
            values[${key}_${t}]="${tag[$t]}"
          done
          eval "$line"
          if [ -n "$values[$key]" ]; then
            n=0
            while [ -n "$values[$key$n]" ]; do n=$(($n + 1)); done
            key=$key$n
          fi
          unset value
          unset tag
        fi
      ;;
    esac
  done
}

view_card() {  #Parameter: Cardfile
  id="$1"
  cardfile="$_DATA/vcard/${id}"
  cachefile="$_DATA/cache/${id}.cache"
  if [ "$cachefile" -nt "$cardfile" ]; then
    cat "$cachefile"
  else
    declare -A values
    vcf_parse "$cardfile"
    . $view_card |tee "$cachefile"
  fi
}

edit_card() {  #Parameter: Cardfile
  id="$1"
  cardfile="$_DATA/vcard/$id"
  tempfile="$_DATA/temp/$id"
  [ -f "$tempfile" ] && cardfile="$tempfile"

  declare -A values
  vcf_parse "$cardfile"
  . $edit_card
}
